"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ColumnDataType_1 = require("./ColumnDataType");
exports.getFieldsFromList = (list, columns) => {
    const fields = {};
    for (const row of list) {
        const fieldNames = Object.keys(row);
        for (const fieldName of fieldNames) {
            if (!fields[fieldName] && columns[fieldName]) {
                fields[fieldName] = columns[fieldName];
            }
        }
    }
    return fields;
};
var ValueType;
(function (ValueType) {
    ValueType["Boolean"] = "boolean";
    ValueType["String"] = "string";
    ValueType["Integer"] = "integer";
    ValueType["Float"] = "float";
    ValueType["Null"] = "null";
    ValueType["Object"] = "object";
})(ValueType = exports.ValueType || (exports.ValueType = {}));
exports.getValueType = (value) => {
    if (typeof (value) === "undefined" || value === null) {
        return ValueType.Null;
    }
    if (typeof (value) === "boolean") {
        return ValueType.Boolean;
    }
    if (typeof (value) === "string") {
        return ValueType.String;
    }
    if (typeof (value) === "number") {
        if (Math.round(value) === value) {
            return ValueType.Integer;
        }
        return ValueType.Float;
    }
    return ValueType.Object;
};
exports.prepareValue = (value, dataType, fieldName, row) => {
    const errorAdd = (fieldName ? ", field: " + fieldName : "") + (row ? ", row: " + JSON.stringify(row) : "");
    const type = exports.getValueType(value);
    switch (dataType) {
        case ColumnDataType_1.default.datetime:
        case ColumnDataType_1.default.date:
            let dt;
            if (type === ValueType.String || type === ValueType.Integer) {
                dt = new Date(value);
            }
            else if (value instanceof Date) {
                dt = value;
            }
            else {
                throw new Error("Invalid value for date: " + value + errorAdd);
            }
            return dt;
        case ColumnDataType_1.default.bigint:
        case ColumnDataType_1.default.int:
        case ColumnDataType_1.default.smallint:
        case ColumnDataType_1.default.mediumint:
        case ColumnDataType_1.default.tinyint:
            let int;
            if (type === ValueType.String) {
                int = parseInt(value, 10);
                if (int.toString() !== value) {
                    throw new Error("Invalid integer value " + value + errorAdd);
                }
            }
            else if (type === ValueType.Integer) {
                int = value;
            }
            else if (type === ValueType.Boolean) {
                int = value ? 1 : 0;
            }
            else {
                throw new Error("Invalid value for integer: " + value + errorAdd);
            }
            return int;
        case ColumnDataType_1.default.float:
        case ColumnDataType_1.default.decimal:
        case ColumnDataType_1.default.real:
            let out;
            if (type === ValueType.Float || type === ValueType.Integer) {
                out = value;
            }
            else if (type === ValueType.String) {
                out = parseFloat(value);
                if (out.toString() !== value) {
                    throw new Error("Invalid float value: " + value + errorAdd);
                }
            }
            else {
                throw new Error("Invalid float value: " + value + errorAdd);
            }
            return out;
        case ColumnDataType_1.default.varchar:
        case ColumnDataType_1.default.nvarchar:
        case ColumnDataType_1.default.char:
        case ColumnDataType_1.default.nchar:
            let str;
            if (type === ValueType.String) {
                str = value;
            }
            else if (type === ValueType.Integer || type === ValueType.Float) {
                str = value.toString();
            }
            else {
                throw new Error("Invalid string value: " + JSON.stringify(value) + errorAdd);
            }
            return str;
        case ColumnDataType_1.default.text:
        case ColumnDataType_1.default.longtext:
        case ColumnDataType_1.default.tinytext:
        case ColumnDataType_1.default.mediumtext:
            let text;
            if (type === ValueType.String) {
                text = value;
            }
            else if (type === ValueType.Integer || type === ValueType.Float) {
                text = value.toString();
            }
            else if (type === ValueType.Object && (exports.isPlainObject(value) || Array.isArray(value))) {
                text = JSON.stringify(value);
            }
            else {
                throw new Error("Invalid string value: " + JSON.stringify(value) + errorAdd);
            }
            return text;
        case ColumnDataType_1.default.json:
            if (type === ValueType.Object && (!exports.isPlainObject(value) && !Array.isArray(value))) {
                throw new Error("Invalis json value, object not plain " + JSON.stringify(value));
            }
            return JSON.stringify(value);
        case ColumnDataType_1.default.blob:
        case ColumnDataType_1.default.longblob:
        case ColumnDataType_1.default.mediumblob:
        case ColumnDataType_1.default.tinyblob:
            if (type === ValueType.Object && Buffer.isBuffer(value)) {
                return value;
            }
        default:
            if (type === ValueType.Object) {
                throw new Error("Value is object, but not string data type: " + JSON.stringify(value) + errorAdd);
            }
            return value;
    }
};
exports.isObject = (val) => {
    return Object.prototype.toString.call(val) === "[object Object]"
        && typeof val === "object";
};
exports.isObjectObject = (o) => {
    return exports.isObject(o) === true
        && Object.prototype.toString.call(o) === "[object Object]";
};
exports.isPlainObject = (o) => {
    let ctor;
    let prot;
    if (exports.isObjectObject(o) === false) {
        return false;
    }
    ctor = o.constructor;
    if (typeof ctor !== "function") {
        return false;
    }
    prot = ctor.prototype;
    if (exports.isObjectObject(prot) === false) {
        return false;
    }
    if (prot.hasOwnProperty("isPrototypeOf") === false) {
        return false;
    }
    return true;
};
