import DataType from "./ColumnDataType";
import { IColumnInfo, IColumns } from "./columns";
export const getFieldsFromList = (list: any[], columns: IColumns) => {
    const fields: { [index: string]: IColumnInfo } = {};
    for (const row of list) {
        const fieldNames = Object.keys(row);
        for (const fieldName of fieldNames) {
            if (!fields[fieldName] && columns[fieldName]) {
                fields[fieldName] = columns[fieldName];
            }
        }
    }
    return fields;
};
export enum ValueType {
    Boolean = "boolean",
    String = "string",
    Integer = "integer",
    Float = "float",
    Null = "null",
    Object = "object",
}
export const getValueType = (value: any): ValueType => {
    if (typeof (value) === "undefined" || value === null) {
        return ValueType.Null;
    }
    if (typeof (value) === "boolean") {
        return ValueType.Boolean;
    }
    if (typeof (value) === "string") {
        return ValueType.String;
    }
    if (typeof (value) === "number") {
        if (Math.round(value) === value) {
            return ValueType.Integer;
        }
        return ValueType.Float;
    }
    return ValueType.Object;
};
export const prepareValue = (value: any, dataType: DataType, fieldName?: string, row?: any) => {
    const errorAdd = (fieldName ? ", field: " + fieldName : "") + (row ? ", row: " + JSON.stringify(row) : "");
    const type = getValueType(value);
    switch (dataType) {
        case DataType.datetime:
        case DataType.date:
            let dt: Date;
            if (type === ValueType.String || type === ValueType.Integer) {
                dt = new Date(value);
            } else if (value instanceof Date) {
                dt = value;
            } else {
                throw new Error("Invalid value for date: " + value + errorAdd);
            }
            return dt;
        case DataType.bigint:
        case DataType.int:
        case DataType.smallint:
        case DataType.mediumint:
        case DataType.tinyint:
            let int: number;
            if (type === ValueType.String) {
                int = parseInt(value, 10);
                if (int.toString() !== value) {
                    throw new Error("Invalid integer value " + value + errorAdd);
                }
            } else if (type === ValueType.Integer) {
                int = value;
            } else if (type === ValueType.Boolean) {
                int = value ? 1 : 0;
            } else {
                throw new Error("Invalid value for integer: " + value + errorAdd);
            }
            return int;
        case DataType.float:
        case DataType.decimal:
        case DataType.real:
            let out: number;
            if (type === ValueType.Float || type === ValueType.Integer) {
                out = value;
            } else if (type === ValueType.String) {
                out = parseFloat(value);
                if (out.toString() !== value) {
                    throw new Error("Invalid float value: " + value + errorAdd);
                }
            } else {
                throw new Error("Invalid float value: " + value + errorAdd);
            }
            return out;
        case DataType.varchar:
        case DataType.nvarchar:
        case DataType.char:
        case DataType.nchar:
            let str: string;
            if (type === ValueType.String) {
                str = value;
            } else if (type === ValueType.Integer || type === ValueType.Float) {
                str = value.toString();
            } else {
                throw new Error("Invalid string value: " + JSON.stringify(value) + errorAdd);
            }
            return str;
        case DataType.text:
        case DataType.longtext:
        case DataType.tinytext:
        case DataType.mediumtext:
            let text: string;
            if (type === ValueType.String) {
                text = value;
            } else if (type === ValueType.Integer || type === ValueType.Float) {
                text = value.toString();
            } else if (type === ValueType.Object && (isPlainObject(value) || Array.isArray(value))) {
                text = JSON.stringify(value);
            } else {
                throw new Error("Invalid string value: " + JSON.stringify(value) + errorAdd);
            }
            return text;
        case DataType.json:
            if (type === ValueType.Object && (!isPlainObject(value) && !Array.isArray(value))) {
                throw new Error("Invalis json value, object not plain " + JSON.stringify(value));
            }
            return JSON.stringify(value);
        case DataType.blob:
        case DataType.longblob:
        case DataType.mediumblob:
        case DataType.tinyblob:
            if (type === ValueType.Object && Buffer.isBuffer(value)) {
                return value;
            }
        default:
            if (type === ValueType.Object) {
                throw new Error("Value is object, but not string data type: " + JSON.stringify(value) + errorAdd);
            }
            return value;
    }
};
export const isObject = (val: any) => {
    return Object.prototype.toString.call(val) === "[object Object]"
        && typeof val === "object";
};
export const isObjectObject = (o: any) => {
    return isObject(o) === true
        && Object.prototype.toString.call(o) === "[object Object]";
};
export const isPlainObject = (o: any) => {
    let ctor: any;
    let prot: any;
    if (isObjectObject(o) === false) {
        return false;
    }
    ctor = o.constructor;
    if (typeof ctor !== "function") {
        return false;
    }
    prot = ctor.prototype;
    if (isObjectObject(prot) === false) {
        return false;
    }
    if (prot.hasOwnProperty("isPrototypeOf") === false) {
        return false;
    }
    return true;
};
